/*
  worldgen.h
  Copyright 2008 Kyrre Glette, Asbjørn Djupdal and Morten Hartmann

  This file is part of RetroRocket, a gravity game for Nintendo DS.

  RetroRocket is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  RetroRocket is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with RetroRocket.  If not, see <http://www.gnu.org/licenses/>.
*/

///////////////////////////////////////////////////////////////////////////////
//
// Autogenerated games
//
///////////////////////////////////////////////////////////////////////////////

#ifndef WORLDGEN_H
#define WORLDGEN_H

///////////////////////////////////////////////////////////////////////////////

struct FPoint //fractal point
{
  FPoint(int x,int y,int color) { this->x=x;this->y=y;c=color; }
  int x,y;   //position
  //unsigned int c;     //color
  int c;     //color
};

///////////////////////////////////////////////////////////////////////////////

/** Class that has the necessary functions for automatically
    creating a new map, represented as a bitmap that Field() can
    create a field from.
*/
class WorldGen {

private:
  void plotRecursive(u8* buffer,int graininessLevel,int iteration,
                     FPoint ul,FPoint ur,FPoint ll,FPoint lr,
                     //upper left, up.right, lower left, l.right
                     int xSize); 
  void addBorder(u8* buffer,int width,int height);
  void removeThinLinesFilter(u8* buffer,int width,int height);

public:
  int width;
  int height;
  int width_tmp;
  int height_tmp;
  int grainy;
  int seaLevel;
  int players;
  u8 *autogenBuffer;
  vector<RRPoint> platforms;

  WorldGen() {
    autogenBuffer = NULL;
  }
  ~WorldGen() {
    if (autogenBuffer) free (autogenBuffer);
  }

  void createBMField (u8* buffer, int xSize, int ySize, int grainy);

  void generate();

  static void previewAutogen (Menu *menu);
  static void createNewAutogen (Menu *menu);
  static void playAutogen (Menu *menu);
  static void placeAutogen (Menu *menu);
  void removeThinLinesFilter() {
    removeThinLinesFilter (autogenBuffer, width, height);
  }

};

#endif
